# Prediction interface for Cog ⚙️
# https://cog.run/python

from cog import BasePredictor, Input, Path
from typing import List
from modelscope.pipelines import pipeline
from util import read_img_result

img_save_folder = "SaveImages"


class Predictor(BasePredictor):
    def setup(self) -> None:
        self.model = pipeline('my-anytext-task',
                              model='damo/cv_anytext_text_generation_editing', model_revision='v1.1.3')

    def predict(
        self,
        mode: str = Input(description="Select model type", default="text-generation", choices=[
            "text-generation", "text-editing"]),
        prompt: str = Input(description="Input prompt",
                            default='photo of caramel macchiato coffee on the table, top-down perspective, with "Any" "Text" written on it using cream'),
        seed: int = Input(description="Seed", default=66273235,
                          ge=0, le=66273235),
        draw_pos: Path = Input(
            description="Draw Pos", default='https://replicate.delivery/pbxt/LIHKXdjxOWFe7HqP1rliIsghRab48EVQRzGNwQ9RgyO5V03d/gen9.png'),
        ori_image: Path = Input(
            description="Ori Image", default='https://replicate.delivery/pbxt/LIHMZ8cCvmndHNVufiSuKZA4mnokuSOy87cYqhvs4Diei7sL/edit9.png'),
        img_count: int = Input(description="Image Count",
                               default=2, ge=0, le=5),
        ddim_steps: int = Input(description="DDIM Steps",
                                default=20, ge=20, le=100),
        use_fp32: bool = Input(description="Use Fp32", default=False),
        no_translator: bool = Input(
            description="No Translator", default=False),
        strength: float = Input(description="", default=1.0, ge=0.0, le=2.0),
        img_width: int = Input(description="Image Width",
                               default=512, ge=256, le=768),
        img_height: int = Input(
            description="Image Height", default=512, ge=512, le=1024),
        cfg_scale: float = Input(
            description="CFG-Scale", default=9.0, ge=0.1, le=30.0),
        a_prompt: str = Input(description="Added Prompt",
                              default="best quality, extremely detailed,4k, HD, supper legible text,  clear text edges,  clear strokes, neat writing, no watermarks"),
        n_prompt: str = Input(description="Negative Prompt",
                              default="low-res, bad anatomy, extra digit, fewer digits, cropped, worst quality, low quality, watermark, unreadable text, messy words, distorted text, disorganized writing, advertising picture"),
        sort_radio: str = Input(description="Sort Position,position sorting priority", choices=[
            "↕", "↔"], default="↕"),
        revise_pos: bool = Input(
            description="Revise Position",  default=False),
    ) -> List[Path]:

        params = {
            "show_debug": True,
            "image_count": img_count,
            "ddim_steps": ddim_steps,
            "strength": strength,
            "image_width": img_width,
            "image_height": img_height,
            "a_prompt": a_prompt,
            "n_prompt": n_prompt,
            "cfg_scale": cfg_scale,
            "revise_pos": revise_pos,
            "sort_priority": sort_radio,
        }
        input_data = {}
        if mode == "text-generation":
            input_data = {
                "prompt": prompt,
                "seed": seed,
                "draw_pos": str(draw_pos),
                "use_fp32": use_fp32,
                "no_translator": no_translator,
            }
        else:
            input_data = {
                "prompt": prompt,
                "seed": seed,
                "draw_pos": str(draw_pos),
                "ori_image": str(ori_image),
                "use_fp32": use_fp32,
                "no_translator": no_translator,
            }
        print("input-data:", input_data)
        results, rtn_code, rtn_warning, debug_info = self.model(
            input_data, mode=mode, **params)
        if rtn_warning:
            print(rtn_warning)
            return []
        res = []
        if rtn_code >= 0:
            res = read_img_result(results)
            # save_images(results, img_save_folder)
            print(f'Done, result images are saved in: {img_save_folder}')
        return res
